#
# Simulation of Reynold's "Boids", a model for flocking behaviour
#
# 2017, Tobias Kohn
#
from gturtle import *
from random import randint
import math

# Number of "boids" in the Simulation
BOID_NUMBER = 25
# The size of each individual boid
BOIDSIZE = 7
# The minimal distance between boids
MIN_DIST = 10
# The maximum speed of any boid
MAXSPEED = 30

def sin(x):
    "sin(x) where x is in degrees"
    return math.sin(math.radians(x))

def cos(x):
    "cos(x) where x is in degrees"
    return math.cos(math.radians(x))

def initialize_boids(number):
    "Create the boids"
    result = []
    repeat number:
        x = randint(-50, 50)
        y = randint(-50, 50)
        a = randint(0, 360)
        result.append((x, y, cos(a), sin(a)))
    return result

def render_boids(boids):
    "Draw the entire scene with all boids"
    # Do not draw any boids while the mouse is updating...
    while lock:
        delay(5)
    if boids is None:
        return
    clear()
    for (x, y, vx, vy) in boids:
        setPos(x, y)
        dot(BOIDSIZE)
    repaint()
        
def update_boids(boids):
    """Calculate the new position and velocity for each boid."""
    # Skip updating if the mouse was pressed
    if lock:
        return
    N = len(boids)
    # Center of mass
    mx = sum([x for (x, y, vx, vy) in boids])
    my = sum([y for (x, y, vx, vy) in boids])
    # Common velocity
    Vx = sum([vx for (x, y, vx, vy) in boids]) / N
    Vy = sum([vy for (x, y, vx, vy) in boids]) / N
    new_boids = []
    for (x, y, vx, vy) in boids:
        vx += Vx
        vy += Vy
        vx += ((mx - x) / N - x) / 100
        vy += ((my - y) / N - y) / 100
        for (x2, y2, _, _) in boids:
            d = sqrt((x2 - x)**2 + (y2 - y) **2)
            if 0 < d < MIN_DIST:
                vx += (x - x2) / 20
                vy += (y - y2) / 20
        v = vx*vx + vy*vy
        if v > MAXSPEED:
            vx *= MAXSPEED / v
            vy *= MAXSPEED / v
        x += vx
        y += vy
        new_boids.append((x, y, vx, vy))
    return new_boids

@onMousePressed
def mousePress(Mx, My):
    global lock
    # Keep updating and drawing from happening
    lock = True
    N = len(boids)
    mx = sum([x for (x, y, vx, vy) in boids])
    my = sum([y for (x, y, vx, vy) in boids])
    for i in indices(boids):
        x, y, vx, vy = boids[i]
        # Scatter the boids: Make the move away from each other
        vx -= ((mx - x) / N - x) / 10
        vy -= ((my - y) / N - y) / 10
        # Make the boids move away from the mouse
        vx += (x - Mx) / 10
        vy += (y - My) / 10
        x += vx
        y += vy
        boids[i] = (x, y, vx, vy)
    lock = False

lock = False
makeTurtle()
hideTurtle()
enableRepaint(False)
boids = initialize_boids(BOID_NUMBER)
repeat:
    if getKeyCode() != 0:
        break
    boids = update_boids(boids)
    render_boids(boids)
    delay(25)
dispose()